@@ -3,14 +3,16 @@
 # under the same terms as Perl itself.
 
 package HTML::Mason::Component::Subcomponent;
-require 5.004;
-require Exporter;
-@ISA = qw(HTML::Mason::Component);
-@EXPORT = qw();
-@EXPORT_OK = qw();
+
+use strict;
 
 use HTML::Mason::Component;
-use strict;
+
+use vars qw(@ISA);
+
+@ISA = qw(HTML::Mason::Component);
+
+use HTML::Mason::MethodMaker ( read_only => [ qw( name owner ) ] );
 
 #
 # Assign parent and name at runtime
@@ -24,9 +26,7 @@ sub cache_file { return $_[0]->owner->cache_file }
 sub create_time { return $_[0]->owner->create-time }
 sub dir_path { return $_[0]->owner->dir_path }
 sub is_subcomp { 1 }
-sub name { return $_[0]->{name} }
 sub object_file { return $_[0]->owner->object_file }
-sub owner { return $_[0]->{owner} }
 sub parent { return $_[0]->owner->parent }
 sub parent_comp { return $_[0]->{owner} }  # deprecated, replaced by owner
 sub parser_version { return $_[0]->owner->parser_version }
